unit chatstatusunit;

interface

uses
  Classes, SysUtils, mainunit, sdl, guiunit,orthounit, guidrawerunit,
  openglUnit, globalunit, stateunit, gl, glu, guiextraunit, messageunit;


type TChatStatus=class(TEventListener)
 public
  Desktop:TUiDesktop;
  GuiDrawer:TGuiDrawer;
  ChatList:TUiList;
  UserList:TUiList;
  GameList:TUiList;
  Edit:TUiEdit;
  
  waitingResponse:boolean;

  Constructor Create;
  Destructor destroy; override;

  procedure InitGui;
 
  procedure onQuitClick(sender:TUiWidget);
  procedure onEnter(sender:TUiWidget);

  procedure onChatClick(sender:TUiWidget);

  procedure onBuyClick(sender:TUiWidget);
	procedure OnAbortGameWinClick(sender:TUiWidget);
  procedure OnCreateGameWinClick(sender:TUiWidget);
  procedure onSellClick(sender:TUiWidget);
  procedure onGamesClick(sender:TUiWidget);
  procedure onCreateGameClick(sender:TUiWidget);
  
  procedure AddChatList(s:string);
  procedure addGame(id, name, mapname, points, mainuser:string);

  procedure Enter; override;
  procedure Exit; override;
  procedure DoLoop; override;
  procedure KeyDownEvent(key:TSDL_KeyboardEvent);override;
  procedure KeyUpEvent(key:TSDL_KeyboardEvent);override;
  procedure MouseDownEvent(event:TSDL_MouseButtonEvent);override;
  procedure MouseUpEvent(event:TSDL_MouseButtonEvent);override;
  procedure MouseMoveEvent(event:TSDL_MouseMotionEvent);override;
end;

var ChatStatus:TChatStatus;

implementation

uses loginstatusunit, gamecreationstatusunit, messageparserunit, gamestatusunit;

procedure TChatStatus.onQuitClick(sender:TUiWidget);
begin
ChangeListener(LoginStatus);
end;

procedure TChatStatus.OnCreateGameWinClick(sender:TUiWidget);
var w:TUiWindow;
    gamename, mapname, points:string;
begin
w:=TUiWindow(sender.parent);
gamename:=TUiEdit(w.WidgetByName('GameNameEdit')).Text;
mapname:=TUiEdit(w.WidgetByName('MapEdit')).Text;
points:=TUiEdit(w.WidgetByName('PointEdit')).Text;
global.conn.Send(msg(M_CREATE_GAME, [mapname, points, gamename]));

sender.parent.free;

waitingResponse:=true;
end;

procedure TChatStatus.OnAbortGameWinClick(sender:TUiWidget);
begin
sender.parent.free;
end;

procedure TChatStatus.onCreateGameClick(sender:TUiWidget);
var w:TUiWindow;
begin
w:=TUiWindow(LoadWidget('media/creategame.ui', 'CreateGameWindow'));
TUiButton( w.WidgetByName('AbortButton') ).OnClick := OnAbortGameWinClick;
TUiButton( w.WidgetByName('CreateButton') ).OnClick := OnCreateGameWinClick;
desktop.addModal(w);
end;

procedure TChatStatus.onEnter(sender:TUiWidget);
begin
global.conn.Send(msg(M_CHAT,[edit.text]));
edit.text:='';
end;


procedure TChatStatus.onChatClick(sender:TUiWidget);
begin
	TUiLayered(Desktop.WidgetByName('lyd1')).PIndexName := 'pagChat';
end;

procedure TChatStatus.onBuyClick(sender:TUiWidget);
begin
  TUiLayered(Desktop.WidgetByName('lyd1')).PIndexName := 'pagBuy';
end;

procedure TChatStatus.onSellClick(sender:TUiWidget);
begin
  TUiLayered(Desktop.WidgetByName('lyd1')).PIndexName := 'pagSell';
end;

procedure TChatStatus.onGamesClick(sender:TUiWidget);
begin
	TUiLayered(Desktop.WidgetByName('lyd1')).PIndexName := 'pagGames';
end;

procedure TChatStatus.initGui;
var l:TUiLabel;
    button:TUiButton;
    Window:TUiWindow;
    e:TUiEdit;
    layered:TUiLayered;
    page:TUiPage;
    Label1:TUiLabel;
begin

	Desktop:=TUiDesktop.create;

  LoadGui(desktop, 'media/chat.ui');

  ChatList:=TUiList( desktop.WidgetByName('ChatList') );
  UserList:=TUiList( desktop.WidgetByName('UserList') );
  GameList:=TUiList( desktop.WidgetByName('GameList') );
  
  Edit:=TUiEdit( desktop.WidgetByName('ChatEdit') );
  Edit.OnEnter:=OnEnter;

  TUiButton( desktop.WidgetByName('btnChat') ).OnClick := OnChatClick;
  TUiButton( desktop.WidgetByName('btnBuy') ).OnClick := OnBuyClick;
  TUiButton( desktop.WidgetByName('btnSell') ).OnClick := OnSellClick;
  TUiButton( desktop.WidgetByName('btnGames') ).OnClick := OnGamesClick;
  TUiButton( desktop.WidgetByName('btnQuit') ).OnClick := OnQuitClick;
  TUiButton( desktop.WidgetByName('CreateGameButton') ).OnClick := OnCreateGameClick;
end;




constructor TChatStatus.create;
begin
InitGui;
GuiDrawer:=TGuiDrawer.Create(global.man);
end;

destructor TChatStatus.Destroy;
begin
Desktop.Free;
GuiDrawer.Free;
end;

procedure TChatStatus.Enter;
begin
Desktop.ResetFocus;
TUiLayered(Desktop.WidgetByName('lyd1')).PIndexName := 'pagChat';
waitingResponse:=false;
end;

procedure TChatStatus.Exit;
begin
end;

procedure TChatStatus.AddChatList(s:string);
begin
	ChatList.Strings.Add(s);
	if ChatList.Strings.count>40 then ChatList.Strings.delete(0);
end;

procedure TChatStatus.addGame(id, name, mapname, points, mainuser:string);
begin
GameList.Strings.add(id+' '+name+'  MAP: '+mapname+' POINTS: '+points+' USER: '+mainuser);
if mainuser = global.username then
 begin
  waitingResponse:=False;
  ChangeListener(GameCreationStatus);
 end;
end;

procedure TChatStatus.DoLoop;
begin
  if global.conn.DataAvaiable then parseMessage;


  // clear z and color
  glClear( GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT );


  // start calculation of the matrix (by loading identity)
  glLoadIdentity;
  gluLookAt(20,20,20,0,0,0,0,1,0);

  // draw something to show gui integration with scene rendering
(*  drawOriginLines(10);
  glColor3f(1,1,1);
//  glRotatef(rot, 0, 1, 0);
  global.man.getTextureByName('car0').bind;
  global.man.getMeshByName('chassis').draw;
*)

  // switch to ortho projection
  BeginOrtho(SCREEN_WIDTH, SCREEN_HEIGHT);

  global.man.getTextureByName('background').bind;
  DrawRectangle(0,0,SCREEN_WIDTH, SCREEN_HEIGHT, 1,1,1);

  // draw the gui
  GuiDrawer.draw(Desktop);
  // draw the mouse pointer
  if waitingResponse then
    global.man.getTextureByName('hourglass').bind
  else
    global.man.getTextureByName('pointer').bind;
  DrawRectangle(global.mx,global.my,64,64, 1, 1, 1);

  // go back to normal projection
  EndOrtho;

  // finally, show our nice frame!
  SDL_GL_SwapBuffers;


end;

procedure TChatStatus.MouseDownEvent(event:TSDL_MouseButtonEvent);
begin
 desktop.DoMouseDown(event.x,event.y, TMouseButton(event.button));
end;

procedure TChatStatus.MouseUpEvent(event:TSDL_MouseButtonEvent);
begin
 desktop.DoMouseUp(event.x,event.y, TMouseButton(event.button));
end;

procedure TChatStatus.MouseMoveEvent(event:TSDL_MouseMotionEvent);
begin
 desktop.DoMouseMove(event.x,event.y);
 global.mx:=event.x;
 global.my:=event.y;
end;

procedure TChatStatus.KeyDownEvent(key:TSDL_KeyboardEvent);
begin
 Desktop.DoKeyPress(char(key.keysym.unicode and $FF));
 global.keystate.processKeyDown(key.keysym.sym);
 if key.keysym.sym = SDLK_ESCAPE then global.loop.StopLoop;
end;

procedure TChatStatus.KeyUpEvent(key:TSDL_KeyboardEvent);
begin
 global.keystate.processKeyUp(key.keysym.sym);
end;



end.

