unit gamestatusunit;

interface

uses
  Classes, SysUtils, mainunit, sdl, guiunit,orthounit, guidrawerunit, modelunit, drawmodelunit,  
  openglUnit, globalunit, stateunit, gl, glu, guiextraunit, messageunit, mapunit, glmapunit, types3d;


type


TCamera = record
  x,y,z:tfloat;
  angley:tfloat;
end;



TGameStatus=class(TEventListener)
 public
  Desktop:TUiDesktop;
  GuiDrawer:TGuiDrawer;
  m1,m2,m3:TModel;
  anim:TRuntimeAnimator;
  animtime:tfloat;
  
  map:TMap;
  
  camera:TCamera;
  
  // widgets
  label1:TUiLabel;


  Constructor Create;
  Destructor destroy; override;

  procedure InitGui;
  
  procedure onExitButtonClick(sender:TUiWidget);
 
  procedure Enter; override;
  procedure Exit; override;
  procedure DoLoop; override;
  procedure KeyDownEvent(key:TSDL_KeyboardEvent);override;
  procedure KeyUpEvent(key:TSDL_KeyboardEvent);override;
  procedure MouseDownEvent(event:TSDL_MouseButtonEvent);override;
  procedure MouseUpEvent(event:TSDL_MouseButtonEvent);override;
  procedure MouseMoveEvent(event:TSDL_MouseMotionEvent);override;
end;

var GameStatus:TGameStatus;

implementation

uses chatstatusunit;


Constructor TGameStatus.Create;
begin
InitGui;
GuiDrawer:=TGuiDrawer.Create(global.man);
m1:=TModel.create;
m1.load('media/tree.model', global.man);
m2:=TModel.create;
m2.load('media/viking_weapon.model', global.man);
m3:=TModel.create;
m3.load('media/tank_ok.model', global.man);
anim:=TRuntimeAnimator.create;
anim.build(m2);

map:=TGlMap.create;

with camera do
 begin
  x:=32;
  y:=20;
  z:=36;
  angley:=45;

 end;
end;

Destructor TGameStatus.destroy;
begin
Desktop.Free;
GuiDrawer.Free;
end;

procedure TGameStatus.initGui;
var l:TUiLabel;
    button:TUiButton;
    Window:TUiWindow;
    i:TUiImage;
begin

  desktop:=TUiDesktop.create;

  LoadGui(desktop, 'media/game.ui');

  TUiButton(desktop.WidgetByName('ExitButton')).OnClick:=onExitButtonClick;
  label1:=TUiLabel.create;
  with label1 do
   begin
    Left:=10;
    top:=580;
    width:=200;
    height:=8;
    caption:='0,0';
   end;
   desktop.Add(label1);
end;




procedure TGameStatus.Enter;
begin
Desktop.ResetFocus;
end;

procedure TGameStatus.Exit;
begin
end;

procedure TGameStatus.onExitButtonClick(sender:TUiWidget);
begin
changeListener(ChatStatus);
end;

procedure TGameStatus.DoLoop;
begin
 if global.mx=0 then
  begin
   camera.x:=camera.x-0.3*cos(camera.angley*pi/180);
   camera.z:=camera.z-0.3*sin(camera.angley*pi/180);
  end;
 if global.mx=799 then
  begin
   camera.x:=camera.x+0.3*cos(camera.angley*pi/180);
   camera.z:=camera.z+0.3*sin(camera.angley*pi/180);
  end;
 if global.my=0 then
  begin
   camera.x:=camera.x+0.3*sin(camera.angley*pi/180);
   camera.z:=camera.z-0.3*cos(camera.angley*pi/180);
  end;
 if global.my=599 then
  begin
   camera.x:=camera.x-0.3*sin(camera.angley*pi/180);
   camera.z:=camera.z+0.3*cos(camera.angley*pi/180);
  end;
  
 label1.Caption:=floattostr(camera.x)+' '+floattostr(camera.z);



  // clear z and color
  glClear( GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT );


  // start calculation of the matrix (by loading identity)
  glLoadIdentity;

  with camera do
   begin
    glRotatef(45,1,0,0);
    glRotatef(angley,0,1,0);
    glTranslatef(-x,-y,-z);

   end;
  
  glLightfv(GL_LIGHT1, GL_POSITION, @LightPosition);

(*  // draw something to show gui integration with scene rendering
  drawOriginLines(10);
  glColor3f(1,1,1);
//  glRotatef(rot, 0, 1, 0);
  global.man.getTextureByName('car0').bind;
  global.man.getMeshByName('chassis').draw;
*)

  glEnable(GL_LIGHTING);
  
  global.man.getTextureByName('tile0').bind;
  map.Render;
  
  
  glTranslatef(2,0,2); // this centers on the first tile
  
  
  glTranslatef(36,0,16);
  drawmodel(m2);
  glTranslatef(4,0,0);
  drawmodel(m1);
  glTranslatef(8,0,8);
  drawmodel(m3);
  
  
  animtime:=animtime+0.003;
  if animtime>=1 then animtime:=animtime-1;
  anim.Interpolate(2, animtime);
  glTranslatef(4,0,4);
  DrawRuntimeModel(anim);

  glDisable(GL_LIGHTING);
  glenable(GL_TEXTURE_2D);

  // switch to ortho projection
  BeginOrtho(SCREEN_WIDTH, SCREEN_HEIGHT);


  // draw the gui
  GuiDrawer.draw(Desktop);
  // draw the mouse pointer
  global.man.getTextureByName('pointer').bind;
  DrawRectangle(global.mx,global.my,64,64, 1, 1, 1);

  // go back to normal projection
  EndOrtho;

  // finally, show our nice frame!
  SDL_GL_SwapBuffers;


end;

procedure TGameStatus.MouseDownEvent(event:TSDL_MouseButtonEvent);
begin
 desktop.DoMouseDown(event.x,event.y, TMouseButton(event.button));
end;

procedure TGameStatus.MouseUpEvent(event:TSDL_MouseButtonEvent);
begin
 desktop.DoMouseUp(event.x,event.y, TMouseButton(event.button));
end;

procedure TGameStatus.MouseMoveEvent(event:TSDL_MouseMotionEvent);
begin
 desktop.DoMouseMove(event.x,event.y);
 global.mx:=event.x;
 global.my:=event.y;
end;

procedure TGameStatus.KeyDownEvent(key:TSDL_KeyboardEvent);
begin
 Desktop.DoKeyPress(char(key.keysym.unicode and $FF));
 global.keystate.processKeyDown(key.keysym.sym);
 if key.keysym.sym = SDLK_ESCAPE then changeListener(ChatStatus);
 if key.keysym.sym = SDLK_LEFT then camera.angley:=camera.angley+1;
 if key.keysym.sym = SDLK_UP then camera.y:=camera.y+1;
 if key.keysym.sym = SDLK_DOWN then camera.y:=camera.y-1;
end;

procedure TGameStatus.KeyUpEvent(key:TSDL_KeyboardEvent);
begin
 global.keystate.processKeyUp(key.keysym.sym);
end;



end.

