unit maineditor;

(*

This unit was written by Nicola {MSX} Lugato.

http://msx.rules.it
nicola@lugato.net
nicola.lugato@gmail.com

It is released under the term of
The GNU General Public License as published by the Free
Software Foundation.

This program is dedicated to Fabiana

Description:

This is the main file of the game.
The TProg class represent the program.

*)

{$M+}

interface


uses
  openglmainloop, screenunit, openglunit, meshunit,
  SDL, GLU, GL, mediamanagerunit,simpleguidrawerunit,
  types3d, SysUtils, guiunit, modelunit, drawmodelunit,
  glmeshunit, orthounit, guiextraunit, textureunit,
  math, msxmath, editorutils, drawmodelpivotunit,
  keyboardunit;

type

TEditMode = (emModel, emPoses, emAnim);

// this class simply represent the program
TProg = class
 public
  keystate:TKeyState;       // keyboard state
  screen:TScreen;           // the opengl screen
  Loop:TOpenGlGenericLoop;   // the opengl game loop
  man:TGenericMediaManager; // the media manager
  desktop:TUiDesktop;
  guidrawer:TGuiDrawer;
  
  
  mx, my:integer;
  
  distance:tfloat;
   
  ax, ay:integer;
  anchor:TMouseButton;
  
  rx,ry:integer;
  
  model, reference:TModel;
  
  animator:TRuntimeAnimator;
  
  Selected:TModelTreeNode;
  SelectedPose:TPose;
  SelectedAnim:TAnimation;
  
  EditMode:TEditMode;
  
  // widgets
  ReferenceModelCheck, 
  DrawFloorCheck,
  LightingCheck,
  DrawPivotCheck:TUiCheckBox;
    
  TreeList, PoseList, AnimList: TUiList;
  NodeTextureImage:TUiImage;
  NodeMeshName,
  NodeModelName,
  NodeReferenceName,NodeNameEdit,
  PoseNameEdit:TUiEdit;
  AnimNameEdit:TUiEdit;
  NodeTextureName, AnimItemPoseEdit:TUiEdit;
  NodeTypeList:TUiList;
  PosXTracker,PosYTracker,PosZTracker,
  RotXTracker,RotYTracker,RotZTracker,
  ScaXTracker,ScaYTracker,ScaZTracker: TUiTracker;
  EditModeButton, AddPoseButton, AddAnimButton:TUiButton;
  
  AnimationPositionTracker:TUiTracker;
  
  AnimMultiTracker:TUiMultiTracker;
  
  ColorRedTracker, ColorGreenTracker, ColorBlueTracker:TUiTracker;

  published

  procedure initGl;
  procedure Start;
  procedure Cleanup;

  
  // event handlers
  procedure OnExitButtonClick(sender:TUiWidget);
  procedure OnTreeListClick(sender:TUiWidget);
  procedure OnPoseListClick(sender:TUiWidget);
  procedure OnTrackerChange(sender:TUiWidget);
  procedure OnAddChildButtonClick(sender:TUiWidget);
  procedure OnColorTrackerChange(sender:TUiWidget);
  procedure OnNodeTypeListClick(sender:TUiWidget);
  procedure OnApplyNodeTypeButtonClick(sender:TUiWidget);
  procedure OnSaveModelButtonClick(sender:TUiWidget);
  procedure OnLoadModelButtonClick(sender:TUiWidget);
  procedure OnEditModeButtonClick(sender:TUiWidget);
  procedure OnAddPoseButtonClick(sender:TUiWidget);
  procedure OnAddAnimButtonClick(sender:TUiWidget);
  procedure OnRenamePoseButtonClick(sender:TUiWidget);
  procedure OnRenameAnimButtonClick(sender:TUiWidget);
  procedure OnAddAnimItemButtonClick(Sender:TUiWidget);
  procedure OnAnimListClick(sender:TUiWidget);
  procedure OnAnimMultiTrackerChange(sender:TUiWidget);
  procedure OnAnimMultiTrackerSelect(sender:TUiWidget);
  procedure OnAnimPoseNameButtonClick(sender:TUiWidget);
  procedure OnAnimationPositionTrackerChange(sender:TUiWidget);

  // ui
  procedure Error(m:string);

  // logic
  procedure BuildModel;
  procedure TrackerToNode;
  procedure NodeToTracker(transf:TTransformation);
  procedure ModelToList;
  procedure ShowSelectedNode;
  procedure ApplyNodeType;
  procedure CycleEditMode;
  procedure ShowPoseList;
  procedure ShowAnimList;
  procedure ShowAnimKey(item:TAnimationKey);
  procedure ShowAnimationDefinition;
  

  procedure DoLoop; 
  procedure KeyDownEvent(key:TSDL_KeyboardEvent);
  procedure KeyUpEvent(key:TSDL_KeyboardEvent);
  procedure MouseDownEvent(event:TSDL_MouseButtonEvent);
  procedure MouseUpEvent(event:TSDL_MouseButtonEvent);
  procedure MouseMoveEvent(event:TSDL_MouseMotionEvent);

  
  
end;



implementation

procedure TProg.OnAnimationPositionTrackerChange(sender:TUiWidget);
begin
  if SelectedAnim=nil then exit;
  // the user changed the animation tracker, so update the animation
  animator.interpolate(AnimList.SelIndex, AnimationPositionTracker.position/1000);
end;

procedure TProg.OnAnimPoseNameButtonClick(sender:TUiWidget);
begin
 // user is changing the pose of an animation key
 if AnimMultiTracker.SelIndex=-1 then
   begin
    error('Select an animation key first!');
    exit;
   end;
 if not model.hasPose(AnimItemPoseEdit.Text) then
   begin
    error('No such pose!');
    exit;
   end;

 // change the pose
 TAnimationKey( AnimMultiTracker.Cursors[AnimMultiTracker.SelIndex].Data ).posename:=AnimItemPoseEdit.Text;
 // rebuild the animation
 animator.Build(model);
 animator.interpolate(AnimList.SelIndex, AnimationPositionTracker.position/1000);
end;

procedure TProg.ShowAnimKey(item:TAnimationKey);
begin
AnimItemPoseEdit.text:=item.poseName;
end;

procedure TProg.OnAnimMultiTrackerChange(sender:TUiWidget);
begin
with TUiMultiTracker(sender) do
 begin
   TAnimationKey(cursors[selindex].Data).time:=cursors[selindex].position;
 
   showAnimKey(TAnimationKey(cursors[selindex].Data));
   animator.Build(model);
   animator.interpolate(AnimList.SelIndex, AnimationPositionTracker.position/1000);
 end;
end;

procedure TProg.OnAnimMultiTrackerSelect(sender:TUiWidget);
begin
with TUiMultiTracker(sender) do
 ShowAnimKey(TAnimationKey(cursors[selindex].Data));
end;

procedure TProg.OnAddAnimItemButtonClick(Sender:TUiWidget);
var i:TAnimationKey;
begin
if model.Poses.Count=0 then
 begin
  error('You must define at least a pose!');
  exit;
 end;
 
if SelectedAnim = nil then
 begin
  error('Select an animation first!');
  exit;
 end;

i:=TAnimationKey.Create;
i.time:=0;
i.posename:=model.Poses[0].name;
SelectedAnim.Keys.Add(i);
 
ShowAnimationDefinition;
end;


procedure TProg.CycleEditMode;
begin
if editmode = high(TEditMode) then editmode:=low(TEditMode) else inc(editmode);
case editmode of
  emModel:
    begin
      EditModeButton.caption:='model';
      Desktop.WidgetByName('NodeColorWindow').Visible:=true;
      Desktop.WidgetByName('NodeTypeWindow').Visible:=true;
      Desktop.WidgetByName('PoseListWindow').Visible:=false;
      Desktop.WidgetByName('NodeTreeWindow').Visible:=true;
      Desktop.WidgetByName('NodeTransformationWindow').Visible:=true;
      Desktop.WidgetByName('AnimListWindow').Visible:=false;
      Desktop.WidgetByName('AnimDefWindow').Visible:=false;
    end;
  emPoses:
    begin
      EditModeButton.caption:='poses';
      Desktop.WidgetByName('NodeColorWindow').Visible:=false;
      Desktop.WidgetByName('NodeTypeWindow').Visible:=false;
      Desktop.WidgetByName('PoseListWindow').Visible:=true;
      Desktop.WidgetByName('NodeTreeWindow').Visible:=true;
      Desktop.WidgetByName('NodeTransformationWindow').Visible:=true;
      Desktop.WidgetByName('AnimListWindow').Visible:=false;
      Desktop.WidgetByName('AnimDefWindow').Visible:=false;
    end;
  emAnim:
    begin
      EditModeButton.caption:='anims';
      Desktop.WidgetByName('NodeColorWindow').Visible:=false;
      Desktop.WidgetByName('NodeTypeWindow').Visible:=false;
      Desktop.WidgetByName('PoseListWindow').Visible:=true;
      Desktop.WidgetByName('NodeTreeWindow').Visible:=false;
      Desktop.WidgetByName('NodeTransformationWindow').Visible:=false;
      Desktop.WidgetByName('AnimListWindow').Visible:=true;
      Desktop.WidgetByName('AnimDefWindow').Visible:=true;
    end;
end;
end;

procedure TProg.ShowPoseList;
var i:integer;
begin
PoseList.Strings.clear;
for i:=0 to model.poses.Count-1 do
 begin
   PoseList.Strings.AddObject(model.poses[i].name, model.poses[i]);
 end;
end;

procedure TProg.ShowAnimList;
var i:integer;
begin
AnimList.Strings.clear;
for i:=0 to model.Animations.Count-1 do
 begin
   AnimList.Strings.AddObject(model.Animations[i].name, model.Animations[i]);
 end;
end;


procedure TProg.OnAddPoseButtonClick(sender:TUiWidget);
begin
model.poses.Add(TPose.create);
ShowPoseList;
end;

procedure TProg.OnAddAnimButtonClick(sender:TUiWidget);
begin
model.Animations.Add(TAnimation.create);
ShowAnimList;
end;

procedure TProg.ShowAnimationDefinition;
var i:integer;
    c:TCursor;
begin
  AnimNameEdit.text:=SelectedAnim.name;
  AnimMultiTracker.Cursors.Clear;
  AnimMultiTracker.SelIndex:=-1;
  for i:=0 to SelectedAnim.KeyS.Count-1 do
    begin
      c:=AnimMultiTracker.AddCursor;
      c.Position:=SelectedAnim.Keys[i].time;
      c.Data:=SelectedAnim.Keys[i];
    end;
  animator.build(model);
  animator.interpolate(AnimList.SelIndex, AnimationPositionTracker.position/1000);
end;

procedure TProg.OnAnimListClick(sender:TUiWidget);
begin
 if TUiList(sender).Strings.Count=0 then exit;

 with TUiList(sender) do
   SelectedAnim:= TAnimation(Strings.Objects[SelIndex]);
   
 ShowAnimationDefinition;

end;



procedure TProg.OnPoseListClick(sender:TUiWidget);
begin
 if TUiList(sender).Strings.Count=0 then exit;
 
 with TUiList(sender) do
   SelectedPose:= TPose(Strings.Objects[SelIndex]);
 poseNameEdit.text:=SelectedPose.name;
// if selected<>nil then ShowSelectedNode;
end;

procedure TProg.OnRenamePoseButtonClick(sender:TUiWidget);
begin
 if SelectedPose=nil then error('Select a pose first!')
 else selectedPose.name:=poseNameEdit.text;

 ShowPoseList;
end;

procedure TProg.OnRenameAnimButtonClick(sender:TUiWidget);
begin
 // @todo move this on global anim "apply"


 if SelectedAnim=nil then error('Select an animation first!')
 else selectedAnim.name:=AnimNameEdit.text;

 ShowAnimList;
end;

procedure TProg.OnEditModeButtonClick(sender:TUiWidget);
begin
CycleEditMode;
end;

procedure TProg.OnSaveModelButtonClick(sender:TUiWidget);
var s:string;
begin
 s:=TUiEdit(Desktop.WidgetByName('ModelNameEdit')).Text;
 model.name:=s;
 if model.Save('media/'+s+'.model') then
  Error('Save Succesfully')
 else
  Error('Error while saving!')
end;

procedure TProg.OnLoadModelButtonClick(sender:TUiWidget);
var s:string;
begin
 // avoid loosing work if you click Load for mistake
 model.save('media/autosave.model');

 s:=TUiEdit(Desktop.WidgetByName('ModelNameEdit')).Text;
 if model.Load('media/'+s+'.model', man) then
 begin
   Selected:=nil;
   SelectedPose:=nil;
   SelectedAnim:=nil;
   AnimList.SelIndex:=-1;
   PoseList.SelIndex:=-1;
   TreeList.SelIndex:=-1;

   ModelToList;
   ShowPoseList;
   ShowAnimList;
   
   Error('Load Succesfully');
 end
 else
  Error('Error while Loading!')

end;

procedure TProg.OnNodeTypeListClick(sender:TUiWidget);
var l:TUiLayered;
begin
l:=TUiLayered(desktop.WidgetByName('NodeTypeLayered'));
case TUiList(sender).selIndex of
 0: l.PIndexName:='NoneTypePage';
 1: l.PIndexName:='MeshTypePage';
 2: l.PIndexName:='ModelTypePage';
 3: l.PIndexName:='ReferenceTypePage';
end;
end;

procedure TProg.ApplyNodeType;
var m:TMesh;
    t:TTexture;
begin
Selected.name:=lowercase(NodeNameEdit.text);
case NodeTypeList.SelIndex of
 0:
   begin
     Selected.GraphicalElement.type_:=geNone;
     Selected.texture:=nil;
   end;
 1:
   begin
     m:=man.getMeshByName(NodeMeshName.Text);
     if m=nil then
      begin
        Error('Mesh '+NodeMeshName.Text+' not found');
        exit;
      end;
      
     if NodeTextureName.Text='' then
      t:=nil
     else
      begin
       t:=man.getTextureByName(NodeTextureName.Text);
       if t=nil then
        begin
          Error('Texture '+NodeTextureName.Text+' not found');
          exit;
        end;
      end;

     Selected.GraphicalElement.type_:=geMesh;
     Selected.GraphicalElement.mesh:=m;
     Selected.texture:=t;
     
     ShowSelectedNode;

   end;
 2:
   begin
     Error('Model not supported yet');
   end;
 3:
   begin
     Error('Reference not supported yet');
   end;

end;
ModelToList;

end;

procedure TProg.OnApplyNodeTypeButtonClick(sender:TUiWidget);
begin
 if selected = nil then
  begin
   Error('Select a node first!');
  end
 else
  begin
    ApplyNodeType;
  end;

end;

procedure TProg.TrackerToNode;
var t:TTransformation;
begin
 if EditMode=emModel then
   begin
     if selected = nil then
       begin
         Error('Select a node first!');
         exit;
       end;
     t:=Selected.transf;
   end;
 if EditMode=emPoses then
   begin
     if selected = nil then
       begin
         Error('Select a node first!');
         exit;
       end;
     if SelectedPose = nil then
       begin
         Error('Select a pose first!');
         exit;
       end;
     t:=SelectedPose.getTransfForNode(selected.name, true); // very empiric!
   end;

 t.Pos.x:=PosXTracker.Position / 100;
 t.Pos.y:=PosYTracker.Position / 100;
 t.Pos.z:=PosZTracker.Position / 100;

 t.Rot.x:=RotXTracker.Position;
 t.Rot.y:=RotYTracker.Position;
 t.Rot.z:=RotZTracker.Position;

 t.Sca.x:=ScaXTracker.Position / 10;
 t.Sca.y:=ScaYTracker.Position / 10;
 t.Sca.z:=ScaZTracker.Position / 10;


end;

procedure TProg.NodeToTracker(transf:TTransformation);
var t:TTransformation;
begin
 // t:=selected.transf;
 if transf =nil then exit;
 t:=transf;
 PosXTracker.Position := Trunc( t.Pos.x * 100 );
 PosYTracker.Position := Trunc( t.Pos.y * 100 );
 PosZTracker.Position := Trunc( t.Pos.z * 100 );

 RotXTracker.Position := Trunc( t.Rot.x );
 RotYTracker.Position := Trunc( t.Rot.y );
 RotZTracker.Position := Trunc( t.Rot.z );

 ScaXTracker.Position := Trunc( t.Sca.x * 10 );
 ScaYTracker.Position := Trunc( t.Sca.y * 10 );
 ScaZTracker.Position := Trunc( t.Sca.z * 10 );
end;

procedure TProg.Error(m:string);
begin
desktop.AddModal(TUiErrorWindow.Create(m));
end;

procedure TProg.OnAddChildButtonClick(sender:TUiWidget);
var a:TModelTreeNode;
begin
 if selected = nil then
  begin
   Error('Select a node first!');
  end
 else
  begin
   a:=TModelTreeNode.create;
   a.name:='New';
   a.texture:=man.getTextureByName('car0');
   a.GraphicalElement.type_:=geMesh;
   a.GraphicalElement.mesh:=man.getMeshByName('chassis');
   selected.Children.Add(a);
   ModelToList;
  end;
end;

{** Shows all data of the selected node into relevant windows }
procedure TProg.ShowSelectedNode;
var l:TUiLayered;
begin

ColorRedTracker.Position:=trunc(selected.color[0]*255);
ColorGreenTracker.Position:=trunc(selected.color[1]*255);
ColorBlueTracker.Position:=trunc(selected.color[2]*255);


 NodeNameEdit.text:=Selected.name;

 l:=TUiLayered(desktop.WidgetByName('NodeTypeLayered'));

 if editmode=emModel then NodeToTracker(selected.transf);
 if editmode=emPoses then
   if (selected<>nil) and (selectedpose<>nil) then
     NodeToTracker(selectedPose.getTransfForNode(selected.name, true));


 NodeTypeList.SelIndex:=integer(selected.GraphicalElement.type_);
 case selected.GraphicalElement.type_ of
   geNone:
     begin
       l.PIndexName:='NoneTypePage';
     end;
   geMesh:
     begin
       NodeMeshName.text:=selected.GraphicalElement.mesh.getName();
       if selected.texture<>nil then
         begin
           NodeTextureName.text:=selected.texture.name;
           NodeTextureImage.Image:=selected.texture.texture;
         end
       else
         begin
           NodeTextureName.text:='';
           NodeTextureImage.Image:=0;
         end;
       l.PIndexName:='MeshTypePage';
     end;
   geModel:
     begin
       NodeModelName.text:='Model';
       l.PIndexName:='ModelTypePage';
     end;
   geReference:
     begin
       NodeReferenceName.text:='1';
       l.PIndexName:='ReferenceTypePage';
     end;
 end;
end;

procedure TProg.OnColorTrackerChange(sender:TUiWidget);
begin
if selected = nil then exit;
selected.color[0]:=ColorRedTracker.Position/255;
selected.color[1]:=ColorGreenTracker.Position/255;
selected.color[2]:=ColorBlueTracker.Position/255;
end;

procedure TProg.OnTrackerChange(sender:TUiWidget);
begin
 TrackerToNode;
end;

procedure TProg.OnTreeListClick(sender:TUiWidget);
begin
 with TUiList(sender) do
   Selected:= TModelTreeNode(Strings.Objects[SelIndex]);
 ShowSelectedNode;
end;

procedure TProg.OnExitButtonClick(sender:TUiWidget);
begin
 Loop.StopLoop;
end;

procedure TProg.initGl;
begin
	// opengl initialization
  glMatrixMode(GL_MODELVIEW);
  glEnable(gl_Blend);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  glDisable(GL_LIGHTING);
  glClearColor(0.8,1,1,0);
  
 SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY,SDL_DEFAULT_REPEAT_INTERVAL);
 SDL_EnableUnicode(1);

 glEnable(GL_TEXTURE_2D);

 // glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
 glEnable(GL_CULL_FACE);
 glCullFace(GL_Back);
 
 
 glLightfv(GL_LIGHT1, GL_AMBIENT, @LightAmbient);
 glLightfv(GL_LIGHT1, GL_DIFFUSE, @LightDiffuse);
 glLightfv(GL_LIGHT1, GL_POSITION, @LightPosition);
 glEnable(GL_LIGHT1);
 glEnable(GL_LIGHTING);

// glShadeModel(GL_FLAT);

 
end;


procedure TProg.Cleanup;
begin
	screen.finalize;
end;


procedure TProg.DoLoop;
begin
  // clear z and color
  glClear( GL_COLOR_BUFFER_BIT or GL_DEPTH_BUFFER_BIT );


  // start calculation of the matrix (by loading identity)
  glLoadIdentity;
  gluLookAt(0,distance,distance,0,2,0,0,1,0);

  glRotated(ry,1,0,0);
  glRotated(rx,0,1,0);

  glLightfv(GL_LIGHT1, GL_POSITION, @LightPosition);
  drawOriginLines(10);
  glColor3f(1,1,1);

  if LightingCheck.Checked then glEnable(GL_LIGHTING) else glDisable(GL_LIGHTING);

  if editMode=emModel then DrawModel(model);
  
  if editMode=emPoses then
    if selectedPose <> nil then
      DrawModelWithPose(model, selectedPose);
      
  if editMode=emAnim then
    if selectedAnim <> nil then
      begin
        DrawRuntimeModel(animator);
      end;


  if DrawFloorCheck.Checked then
    begin
      glColor3f(1,1,1);
      SetMaterialColor(1,1,1);
      glEnable(GL_TEXTURE_2D);
      man.getTextureByName('tile0').bind;
      Rectangle(-2,-2,4,4);
      glDisable(GL_TEXTURE_2D);
    end;
  
  if ReferenceModelCheck.Checked then
    begin
   	  gltranslatef(4,0,0);
      DrawModel(reference);
      gltranslatef(-4,0,0);
    end;

  glDisable(GL_LIGHTING);

  if DrawPivotCheck.Checked then 
    begin
      glDisable(GL_DEPTH_TEST);
      DrawModelPivot(model);
      glEnable(GL_DEPTH_TEST);
    end;


  // switch to ortho projection
  BeginOrtho(SCREEN_WIDTH, SCREEN_HEIGHT);

  // draw the gui
  GuiDrawer.draw(Desktop);
  // draw the mouse pointer
  man.getTextureByName('pointer').bind;
  DrawRectangle(mx,my,64,64, 1, 1, 1);

  // go back to normal projection
  EndOrtho;

  // finally, show our nice frame!
  SDL_GL_SwapBuffers;


end;

procedure TProg.MouseDownEvent(event:TSDL_MouseButtonEvent);
begin
 desktop.DoMouseDown(event.x,event.y, TMouseButton(event.button));
 if not desktop.IsOverWidget then
  begin
	 anchor:=TMouseButton(event.button);
	 ax:=mx;
	 ay:=my;
  end;
end;

procedure TProg.MouseUpEvent(event:TSDL_MouseButtonEvent);
begin
 if anchor = mbnone then
  desktop.DoMouseUp(event.x,event.y, TMouseButton(event.button))
 else
  begin
   anchor:=mbNone;
  end;
end;

procedure TProg.MouseMoveEvent(event:TSDL_MouseMotionEvent);
begin
 mx:=event.x;
 my:=event.y;

 if anchor=mbNone then
   desktop.DoMouseMove(event.x,event.y)
 else
   begin
		 if anchor=mbLeft then 
		   begin
		   	rx:=rx+mx-ax;
		   	ry:=ry+my-ay;
  			ax:=mx;
	      ay:=my;
		   end;
		 if anchor=mbRight then 
		   begin
		   	distance:=distance+(my-ay) / 5;
		   	distance:=max(distance, 0.01);
				 ax:=mx;
				 ay:=my;
		   end;
  end;
end;

procedure TProg.KeyDownEvent(key:TSDL_KeyboardEvent);
begin
 Desktop.DoKeyPress(char(key.keysym.unicode and $FF));
 keystate.processKeyDown(key.keysym.sym);
 if key.keysym.sym = SDLK_ESCAPE then loop.StopLoop;
end;

procedure TProg.KeyUpEvent(key:TSDL_KeyboardEvent);
begin
 keystate.processKeyUp(key.keysym.sym);
end;

procedure TProg.BuildModel;
var a:TModelTreeNode;
begin
a:=TModelTreeNode.create;
a.name:='Root';
with a.GraphicalElement do
  begin
    type_:=geNone;
  end;
a.texture:= nil;

model.root:=a;

ModelToList;
end;


procedure TProg.ModelToList;

  procedure NodeToList(node:TModelTreeNode; ind:string);
  var i:integer;
  begin
    TreeList.Strings.AddObject(ind+node.name, node);
    ind:=ind+' ';
    for i:=0 to node.children.count-1 do
      nodeToList(node.children[i], ind);
  end;

begin
  TreeList.Strings.Clear;
  NodeToList(model.root, '');
end;

procedure TProg.Start;
var e:TOnSimpleEvent;
    m:TMethod;
begin
// StateManager:=TStateManager.Create;
 
 screen:=TScreen.Create('Model Editor by {MSX}!', 800, 600, 16, ParamStr(1)='-fs', 500);
 if not screen.Initialize then exit;
 
 // keyboard state helper
 keystate:=tkeystate.create;

 // start up our kind media manager
 man:=TFileMediaManager.Create;

 // set up the main loop
 Loop:=TOpenGlStaticLoop.create;
 TOpenGlStaticLoop(loop).CallCycleAfterEvents:=true;

 guidrawer:=TGuiDrawer.create(man);
 
 desktop:=TUiDesktop.create;

 LoadGui(desktop, 'media/editor.ui');
 TUiButton(desktop.WidgetByName('ExitButton')).OnClick:=OnExitButtonClick;
 TreeList := TUiList(desktop.WidgetByName('TreeList'));
 TreeList.OnClick:=OnTreeListClick;
 PoseList := TUiList(desktop.WidgetByName('PoseList'));
 PoseList.OnClick:=OnPoseListClick;


 AnimList:= TUiList(desktop.WidgetByName('AnimList'));
 AnimList.OnClick:=OnAnimListClick;
 
 TUiButton(desktop.WidgetByName('AddChildButton')).OnClick:=OnAddChildButtonClick;

 writeln(integer(TProg.MethodAddress('OnAddChildButtonClick')));

 LightingCheck:=TUiCheckBox(desktop.WidgetByName('LightingCheck'));
 ReferenceModelCheck:=TUiCheckBox(desktop.WidgetByName('ReferenceModelCheck'));
 DrawFloorCheck:=TUiCheckBox(desktop.WidgetByName('DrawFloorCheck'));
 DrawPivotCheck:=TUiCheckBox(desktop.WidgetByName('DrawPivotCheck'));
 EditModeButton:=TUiButton(desktop.WidgetByName('EditModeButton'));
 EditModeButton.OnClick:=OnEditModeButtonClick;
 AddPoseButton:=TUiButton(desktop.WidgetByName('AddPoseButton'));
 AddPoseButton.OnClick:=OnAddPoseButtonClick;

 AddAnimButton:=TUiButton(desktop.WidgetByName('AddAnimButton'));
 AddAnimButton.OnClick:=OnAddAnimButtonClick;


 TUiButton(desktop.WidgetByName('SaveModelButton')).OnClick:=OnSaveModelButtonClick;
 TUiButton(desktop.WidgetByName('LoadModelButton')).OnClick:=OnLoadModelButtonClick;
 TUiButton(desktop.WidgetByName('ApplyNodeTypeButton')).OnClick:=OnApplyNodeTypeButtonClick;
 TUiButton(desktop.WidgetByName('RenamePoseButton')).OnClick:=OnRenamePoseButtonClick;
 TUiButton(desktop.WidgetByName('RenameAnimButton')).OnClick:=OnRenameAnimButtonClick;
 TUiButton(desktop.WidgetByName('AddAnimItemButton')).OnClick:=OnAddAnimItemButtonClick;
 TUiButton(desktop.WidgetByName('AnimPoseNameButton')).OnClick:=OnAnimPoseNameButtonClick;
 NodeTextureImage:=TUiImage(desktop.WidgetByName('NodeTextureImage'));

 NodeMeshName:=TUiEdit(desktop.WidgetByName('NodeMeshName'));
 NodeModelName:=TUiEdit(desktop.WidgetByName('NodeModelName'));
 NodeReferenceName:=TUiEdit(desktop.WidgetByName('NodeReferenceName'));
 NodeNameEdit:=TUiEdit(desktop.WidgetByName('NodeNameEdit'));
 PoseNameEdit:=TUiEdit(desktop.WidgetByName('PoseNameEdit'));
 AnimNameEdit:=TUiEdit(desktop.WidgetByName('AnimNameEdit'));
 AnimItemPoseEdit:=TUiEdit(desktop.WidgetByName('AnimItemPoseEdit'));

 NodeTextureName:=TUiEdit(desktop.WidgetByName('NodeTextureName'));

 NodeTypeList:=TUiList(desktop.WidgetByName('NodeTypeList'));
 NodeTypeList.OnClick:=OnNodeTypeListClick;
 

 AnimMultiTracker:=TUiMultiTracker(desktop.WidgetByName('AnimMultiTracker'));
 AnimMultiTracker.OnChange:=OnAnimMultiTrackerChange;
 AnimMultiTracker.OnSelect:=OnAnimMultiTrackerSelect;

 PosXTracker:=TUiTracker(desktop.WidgetByName('PosX'));
 PosYTracker:=TUiTracker(desktop.WidgetByName('PosY'));
 PosZTracker:=TUiTracker(desktop.WidgetByName('PosZ'));
 RotXTracker:=TUiTracker(desktop.WidgetByName('RotX'));
 RotYTracker:=TUiTracker(desktop.WidgetByName('RotY'));
 RotZTracker:=TUiTracker(desktop.WidgetByName('RotZ'));
 ScaXTracker:=TUiTracker(desktop.WidgetByName('ScaX'));
 ScaYTracker:=TUiTracker(desktop.WidgetByName('ScaY'));
 ScaZTracker:=TUiTracker(desktop.WidgetByName('ScaZ'));
 
 PosXTracker.OnChange:=OnTrackerChange;
 PosYTracker.OnChange:=OnTrackerChange;
 PosZTracker.OnChange:=OnTrackerChange;
 RotXTracker.OnChange:=OnTrackerChange;
 RotYTracker.OnChange:=OnTrackerChange;
 RotZTracker.OnChange:=OnTrackerChange;
 ScaXTracker.OnChange:=OnTrackerChange;
 ScaYTracker.OnChange:=OnTrackerChange;
 ScaZTracker.OnChange:=OnTrackerChange;
 
 
 AnimationPositionTracker:=TUiTracker(desktop.WidgetByName('AnimationPositionTracker'));
 AnimationPositionTracker.onChange:=OnAnimationPositionTrackerChange;
 
 ColorRedTracker:=TUiTracker(desktop.WidgetByName('ColorRedTracker'));
 ColorGreenTracker:=TUiTracker(desktop.WidgetByName('ColorGreenTracker'));
 ColorBlueTracker:=TUiTracker(desktop.WidgetByName('ColorBlueTracker'));

 ColorRedTracker.OnChange:=OnColorTrackerChange;
 ColorGreenTracker.OnChange:=OnColorTrackerChange;
 ColorBlueTracker.OnChange:=OnColorTrackerChange;

 model:=Tmodel.creaTE;
 reference:=Tmodel.creaTE;
 reference.load('media/man.model', man);
 animator:=TRuntimeAnimator.Create;
 BuildModel;
 
(* anim.poses.Add(TPose.create);
 anim.poses[0].Items.Add(TPoseItem.create);
 anim.poses[0].Items[0].movedNode:='shoulderdx';
 anim.poses[0].Items[0].diff.Rot.x:=-90;
 anim.poses[0].Items.Add(TPoseItem.create);
 anim.poses[0].Items[1].movedNode:='shouldersx';
 anim.poses[0].Items[1].diff.Rot.x:=-90;
*)
 // this resets to emModel mode
 EditMode:=high(TEditMode);
 CycleEditMode;

 
 // initialize some other things
 initGl;
 
 
 anchor:=mbNone;
 distance:=10;
 rx:=-45;

 loop.setCycleEvent(DoLoop);
 loop.setMouseDownEvent(MouseDownEvent);
 loop.setMouseUpEvent(MouseUpEvent);
 loop.setMouseMoveEvent(MouseMoveEvent);
 loop.setKeyDownEvent(KeyDownEvent);
 loop.setKeyUpEvent(KeyUpEvent);

 // ready to go! :P
 loop.StartLoop;

end;




end.
