unit guidrawerunit;

interface

uses
  Classes, SysUtils, guiunit, guiextraunit,
  openglunit,
  SDL, GLU, GL, mediamanagerunit,
  types3d, 
  orthounit;

type

TAbstractGuiDrawer = class
  public
    procedure draw(desktop:TUiDesktop); virtual; abstract;
end;


TGuiDrawer = class(TAbstractGuiDrawer)
  private
    fMan:TGenericMediaManager;
    procedure DrawWidget(w:TUiWidget);
    procedure DrawChildren(c:TUiContainer);
    procedure DrawWindow(W:TUiWindow);
    procedure DrawLayered(l:TUiLayered);
    procedure DrawImage(w:TUiImage);
    procedure DrawEdit(w:TUiEdit);
    procedure DrawPage(w:TUiPage);
    procedure DrawLabel(w:TUiLabel);
    procedure DrawButton(w:TUiButton);
    procedure DrawList(w:TUiList);
    procedure DrawTracker(w:TUiTracker);
    procedure DrawCheckBox(w:TUiCheckBox);
    procedure DrawBar(w:TUiBar);
  public
    constructor Create(manager:TGenericMediaManager);
    procedure draw(desktop:TUiDesktop);override;
end;

implementation

var 
    CHAR_WIDTH:integer=8;
    CHAR_HEIGHT:integer=8;

procedure drawChar(x,y:integer; c:char);
var tx,ty,w,h:single;
    i:byte;
begin
  i:=ord(C)-32;
  tx:=((i mod 16)*CHAR_WIDTH)/128;
  ty:=((i div 16)*CHAR_HEIGHT)/128 + 0.5;
  w := CHAR_WIDTH/128;
  h := CHAR_HEIGHT/128;

  glBegin( GL_TRIANGLE_STRIP  );         // Drawing Using Triangles
    glTexCoord2f(tx,ty);
    glVertex2f(  x,  y );
    glTexCoord2f(tx,ty+h);
    glVertex2f(  x,  y+CHAR_HEIGHT );
    glTexCoord2f(tx+w,ty);
    glVertex2f(  x+CHAR_WIDTH,  y );
    glTexCoord2f(tx+w,ty+h);
    glVertex2f(  x+CHAR_WIDTH,  y+CHAR_HEIGHT );
  glEnd;                            // Finished Drawing The Triangles

end;

procedure drawString(x,y:integer; s:string);
var i:integer;
begin
  glColor3f(0,0,0);

  for i:=1 to length(s) do
   drawChar(x+(i-1)*(CHAR_WIDTH-1), y, s[i]);
end;

procedure DrawRectUv25(x,y,w,h:integer; u,v:TFloat);
begin
  glBegin( GL_TRIANGLE_STRIP  );         // Drawing Using Triangles
    glTexCoord2f(u,v);
    glVertex2f(  x,  y );

    glTexCoord2f(u,v+0.25);
    glVertex2f(  x,  y+H );

    glTexCoord2f(u+0.25,v);
    glVertex2f(  x+W,  y );

    glTexCoord2f(u+0.25,v+0.25);
    glVertex2f(  x+W,  y+H );
  glEnd;                            // Finished Drawing The Triangle
end;

procedure DrawRect(x,y,w,h:integer);
begin
drawRectUv25(x,y,16,16, 0,0);
drawRectUv25(x,y+h-16,16,16, 0, 0.5);
drawRectUv25(x+w-16,y,16,16, 0.5, 0);
drawRectUv25(x+w-16,y+h-16,16,16, 0.5, 0.5);

drawRectUv25(x+16,y,w-32,16, 0.25, 0);
drawRectUv25(x+16,y+h-16,w-32,16, 0.25, 0.5);

drawRectUv25(x,y+16,16,h-32, 0, 0.25);
drawRectUv25(x+w-16,y+16,16,h-32, 0.5, 0.25);

drawRectUv25(x+16,y+16,w-32,h-32, 0.25, 0.25);
end;

constructor TGuiDrawer.create(manager:TGenericMediaManager);
begin
	fman:=manager;
end;

procedure TGuiDrawer.Draw(Desktop:TUiDesktop);
var i:integer;
begin
   for i:=0 to Desktop.Count-1 do
     DrawWidget(Desktop[i]);
end;


procedure TGuiDrawer.DrawChildren(c:TUiContainer);
var i:integer;
begin
   glTranslatef( c.Left+c.MarginLeft,c.Top+c.MarginTop, 0);
   for i:=0 to c.Count-1 do
     DrawWidget(c[i]);

   glTranslatef(-c.Left-c.MarginLeft, -c.Top-c.MarginTop, 0);


end;

procedure TGuiDrawer.DrawPage(w:TUiPage);
begin
 if w.DrawBack then
   begin
     if w.enabled then
       fman.getTextureByName('interf4').bind
     else
       fman.getTextureByName('interf4g').bind;
     glColor3f(1, 1, 1);
     DrawRect(w.Left, w.Top, w.Width, w.Height);
   end;
   
 DrawChildren(w);
end;

procedure TGuiDrawer.DrawLayered(l:TUiLayered);
begin
  glTranslatef( l.Left+l.MarginLeft,l.Top+l.MarginTop, 0);
  DrawWidget(l[l.count-1]);//Only draw the widget at the front (ei the active page and all it's children)
  glTranslatef(-l.Left-l.MarginLeft, -l.Top-l.MarginTop, 0);
end;

procedure TGuiDrawer.DrawWindow(W:TUiWindow);
begin
     if w.enabled then
       fman.getTextureByName('interf3').bind
     else
       fman.getTextureByName('interf3b').bind;

if w.Focused then glColor3f(1, 1, 0) else glColor3f(1, 1, 1);

DrawRect(w.Left, w.Top, w.Width, w.Height);

fman.getTextureByName('font').bind;
drawString(w.Left+10, w.Top+5,w.title);

DrawChildren(w);
end;


procedure TGuiDrawer.DrawButton(w:TUiButton);
var i:integer;
    c:TUiContainer;
begin
     if w.enabled then
       fman.getTextureByName('interf4').bind
     else
       fman.getTextureByName('interf4g').bind;


if w.Focused then glColor3f(1, 1, 0) else glColor3f(1, 1, 1);

DrawRect(w.Left, w.Top, w.Width, w.Height);

fman.getTextureByName('font').bind;
if w.clicked then
  drawString(w.Left+12, w.Top+13,w.Caption)
else
  drawString(w.Left+12, w.Top+10,w.Caption);
end;


procedure TGuiDrawer.DrawEdit(w:TUiEdit);
var i:integer;
    c:TUiContainer;
    s:string;
begin
     if w.enabled then
       fman.getTextureByName('interf4').bind
     else
       fman.getTextureByName('interf4g').bind;


if w.Focused then glColor3f(1, 1, 0) else glColor3f(1, 1, 1);

DrawRect(w.Left, w.Top, w.Width, w.Height);

fman.getTextureByName('font').bind;

s:='';
if w.mask then 
 begin
 	 for i:=1 to length(w.text) do s:=s+'*';
 end
else 
 s:=w.text;

if w.focused then s:=s+'_';

drawString(w.Left+12, w.Top+10,s)
  
end;

procedure TGuiDrawer.DrawImage(w:TUiImage);
var i:integer;
begin
//glColor3f(1, 1, 1);

glBindTexture(GL_TEXTURE_2D, w.image);

DrawRectangle(w.Left, w.Top, w.Width, w.Height,1,1,1);

end;

procedure TGuiDrawer.DrawList(w:TUiList);
var i:integer;
    c:TUiContainer;
begin
 if w.enabled then
   fman.getTextureByName('interf4').bind
 else
   fman.getTextureByName('interf4g').bind;

glColor3f(1, 1, 1);

DrawRect(w.Left, w.Top, w.Width, w.Height);

{ draw a bar behind selected text }
if (w.SelIndex>=0) and (not w.Simple) then 
  begin
   glDisable(GL_TEXTURE_2D);
   DrawRectangle(w.Left+10,w.Top+12+(w.SelIndex*12),w.Width-20,13, 0.6, 0.6, 0.6);
   glEnable(GL_TEXTURE_2D);
  end;

fman.getTextureByName('font').bind;
for i:=0 to w.Strings.count-1 do
 drawString(w.Left+10, w.Top+16+i*12,w.strings[i]);

end;


procedure TGuiDrawer.DrawTracker(w:TUiTracker);
var i:integer;
    c:TUiContainer;
    realPos, logicPos, realWidth, logicWidth:integeR;
begin
 if w.enabled then
   fman.getTextureByName('interf4').bind
 else
   fman.getTextureByName('interf4g').bind;

glColor3f(1, 1, 1);

DrawRect(w.Left, w.Top, w.Width, w.Height);

{ draw the cursor  }
logicPos := w.position - w.min;
logicWidth := w.max-w.min;
realwidth	:=w.width-2*w.margin;

realpos:=(logicPos*realwidth div logicwidth) + w.margin;

   glDisable(GL_TEXTURE_2D);
   DrawRectangle(w.Left+realpos-(w.CursorWidth div 2),w.Top,w.cursorwidth,w.height, 0.6, 0.6, 0.6);
   glEnable(GL_TEXTURE_2D);


fman.getTextureByName('font').bind;
drawString(w.Left+12, w.Top+10, inttostr(w.position)); // @todo use w.caption

end;


procedure TGuiDrawer.DrawLabel(w:TUiLabel);
begin
fman.getTextureByName('font').bind;
drawString(w.Left, w.Top,w.Caption);

end;


procedure TGuiDrawer.DrawWidget(w:TUiWidget);
var i:integer;
    c:TUiContainer;
begin
if w is TUiWindow then DrawWindow( TUiWindow(w) )
else if w is TUiButton then DrawButton( TUiButton(w) )
else if w is TUiLayered then DrawLayered( TUiLayered(w) )
else if w is TUiPage then DrawPage( TUiPage(w) )
else if w is TUiEdit then DrawEdit( TUiEdit(w) )
else if w is TUiList then DrawList( TUiList(w) )
else if w is TUiTracker then DrawTracker( TUiTracker(w) )
else if w is TUiImage then DrawImage( TUiImage(w) )
else if w is TUiLabel then DrawLabel( TUiLabel(w) )
else if w is TUiCheckBox then DrawCheckBox( TUiCheckBox(w) )
else if w is TUiBar then DrawBar( TUiBar(w) );


end;

procedure TGuiDrawer.DrawCheckBox(w:TUiCheckBox);
begin
    if w.enabled then
       fman.getTextureByName('interf4').bind
    else
       fman.getTextureByName('interf4g').bind;


if w.Focused then glColor3f(1, 1, 0) else glColor3f(1, 1, 1);

DrawRect(w.Left, w.Top, 32, 32);

if w.Checked then begin
  fman.getTextureByName('interftick').Bind;
  DrawRectangle(w.Left, w.Top, 32, 32,1,1,1);
end;


fman.getTextureByName('font').bind;
drawString(w.Left+34, w.Top+10,w.Caption);
end;


procedure TGuiDrawer.DrawBar(w:TUiBar);
var postext:string;
    BarPos:real;
    pos:real;
begin
{!This whole procedure could do with with a clean up, and optimization!}

//Process the bar to get the correct length
  pos := w.Position;
  if pos > w.Max then Pos := w.Max;             //just some error checking
  if Pos <= 0 then Pos := 1;                   //just some error checking (must be 1, doesn't like dividing by 0)

  BarPos := w.Width / (w.Max / Pos);

//Draw a very simple bar
if w.Position > 0 then begin
  glDisable(GL_TEXTURE_2D);
    DrawRectangle(w.Left+1,w.Top+1,trunc(BarPos)-2,w.Height-2,1, 1, 0);
  glEnable(GL_TEXTURE_2D);
end;

fman.getTextureByName('interf6').bind;

glColor3f(1, 1, 1);
DrawRect(w.Left, w.Top, w.Width, w.Height);

//Draw the extra stuff
if w.ShowCaption then begin
  fman.getTextureByName('font').bind;
  drawString(w.Left+4, w.Top+4,w.Caption);
end;

if w.ShowPosition then begin
  postext := IntToStr(w.Position)+'/'+IntToStr(w.Max);
  fman.getTextureByName('font').bind;
  drawString(                                                       //draw the text centered
    w.Left+((w.Width div 2)-((Length(postext)*CHAR_WIDTH)div 2)),
    w.Top+(w.Height div 2)-(CHAR_HEIGHT div 2),
    postext
  );
end;

end;



end.

