unit simpleguidrawerunit;

interface

uses
  Classes, SysUtils, guiunit, guiextraunit,
  openglunit,
  SDL, GLU, GL, mediamanagerunit,
  types3d, 
  orthounit;

type TRgb = array [0..2] of single;


const 
cNormal:TRgb = (0.7,0.7,0.7);
cDisabled:TRgb = (0.3,0.3,0.3);
cFocused:TRgb = (0.6,0.9,0.9);
cWidget:TRgb = (0.3,0.7,0.7);
cPage:TRgb = (0.3,0.5,0.7);
cEdit:TRgb = (0.7,0.3,0.7);

type

TAbstractGuiDrawer = class
  public
    procedure draw(desktop:TUiDesktop); virtual; abstract;
end;


TGuiDrawer = class(TAbstractGuiDrawer)
  private
    fMan:TGenericMediaManager;
    procedure DrawWidget(w:TUiWidget);
    procedure DrawChildren(c:TUiContainer);
    procedure DrawWindow(W:TUiWindow);
    procedure DrawRollableWindow(W:TUiRollableWindow);
    procedure DrawLayered(l:TUiLayered);
    procedure DrawImage(w:TUiImage);
    procedure DrawEdit(w:TUiEdit);
    procedure DrawPage(w:TUiPage);
    procedure DrawLabel(w:TUiLabel);
    procedure DrawButton(w:TUiButton);
    procedure DrawList(w:TUiList);
    procedure DrawTracker(w:TUiTracker);
    procedure DrawMultiTracker(w:TUiMultiTracker);
    procedure DrawCheckBox(w:TUiCheckBox);
    procedure DrawBar(w:TUiBar);
  public
    constructor Create(manager:TGenericMediaManager);
    procedure draw(desktop:TUiDesktop);override;
end;

implementation

var 
    CHAR_WIDTH:integer=8;
    CHAR_HEIGHT:integer=8;

procedure SetColor(c:TRgb);
begin
	glColor3f(c[0],c[1],c[2]);
end;
procedure DrawRectangle(x,y,w,h:single);
begin
  glBegin( GL_TRIANGLE_STRIP  );         // Drawing Using Triangles
    glTexCoord2f(0,0);
    glVertex2f(  x,  y );

    glTexCoord2f(0,1);
    glVertex2f(  x,  y+H );

    glTexCoord2f(1,0);
    glVertex2f(  x+W,  y );

    glTexCoord2f(1,1);
    glVertex2f(  x+W,  y+H );
  glEnd;                            // Finished Drawing The Triangle
end;

procedure DrawRectangleOutline(x,y,w,h:single);
begin
  glBegin( GL_Line_loop  );         // Drawing Using Triangles
    glVertex2f(  x,  y );

    glVertex2f(  x,  y+H );
    
    glVertex2f(  x+W,  y+H );
    
    glVertex2f(  x+W,  y );
    
    
  glEnd;                            // Finished Drawing The Triangle
end;


procedure drawChar(x,y:integer; c:char);
var tx,ty,w,h:single;
    i:byte;
begin
  i:=ord(C)-32;
  tx:=((i mod 16)*CHAR_WIDTH)/128;
  ty:=((i div 16)*CHAR_HEIGHT)/128 + 0.5;
  w := CHAR_WIDTH/128;
  h := CHAR_HEIGHT/128;

  glBegin( GL_TRIANGLE_STRIP  );         // Drawing Using Triangles
    glTexCoord2f(tx,ty);
    glVertex2f(  x,  y );
    glTexCoord2f(tx,ty+h);
    glVertex2f(  x,  y+CHAR_HEIGHT );
    glTexCoord2f(tx+w,ty);
    glVertex2f(  x+CHAR_WIDTH,  y );
    glTexCoord2f(tx+w,ty+h);
    glVertex2f(  x+CHAR_WIDTH,  y+CHAR_HEIGHT );
  glEnd;                            // Finished Drawing The Triangles

end;

procedure drawString(x,y:integer; s:string);
var i:integer;
begin
	glEnable(GL_TEXTURE_2D);
  glColor3f(0,0,0);

  for i:=1 to length(s) do
   drawChar(x+(i-1)*(CHAR_WIDTH-1), y, s[i]);
  glDisable(GL_TEXTURE_2D);
end;

constructor TGuiDrawer.create(manager:TGenericMediaManager);
begin
	fman:=manager;
end;

procedure TGuiDrawer.Draw(Desktop:TUiDesktop);
var i:integer;
begin
	 glDisable(GL_TEXTURE_2D);
   for i:=0 to Desktop.Count-1 do
     DrawWidget(Desktop[i]);
   glEnable(GL_TEXTURE_2D);
end;


procedure TGuiDrawer.DrawChildren(c:TUiContainer);
var i:integer;
begin
   glTranslatef( c.Left+c.MarginLeft,c.Top+c.MarginTop, 0);
   for i:=0 to c.Count-1 do
     DrawWidget(c[i]);

   glTranslatef(-c.Left-c.MarginLeft, -c.Top-c.MarginTop, 0);


end;

procedure TGuiDrawer.DrawPage(w:TUiPage);
begin
 if w.DrawBack then
   begin
     if w.enabled then setColor(cPage) else setColor(cDisabled);

     DrawRectangle(w.Left, w.Top, w.Width, w.Height);
   end;
   
 DrawChildren(w);
end;

procedure TGuiDrawer.DrawLayered(l:TUiLayered);
begin
  glTranslatef( l.Left+l.MarginLeft,l.Top+l.MarginTop, 0);
  DrawWidget(l[l.count-1]);//Only draw the widget at the front (ei the active page and all it's children)
  glTranslatef(-l.Left-l.MarginLeft, -l.Top-l.MarginTop, 0);
end;

procedure TGuiDrawer.DrawRollableWindow(W:TUiRollableWindow);
begin
if not w.Visible then exit;

if w.open then
 begin
  DrawWindow(w);
  exit;
 end;
 
if w.enabled then setColor(cNormal) else setColor(cDisabled);

setColor(cWidget);
DrawRectangle(w.Left, w.Top, w.Width, w.MarginTop);

glColor3f(0,0,0);
DrawRectangleOutline(w.Left, w.Top, w.Width, w.MarginTop);

fman.getTextureByName('font').bind;
drawString(w.Left+10, w.Top+5,w.title);
end;

procedure TGuiDrawer.DrawWindow(W:TUiWindow);
begin
if w.enabled then setColor(cNormal) else setColor(cDisabled);

DrawRectangle(w.Left, w.Top, w.Width, w.Height);

setColor(cWidget);
DrawRectangle(w.Left, w.Top, w.Width, w.MarginTop);

glColor3f(0,0,0);
DrawRectangleOutline(w.Left, w.Top, w.Width, w.height);

fman.getTextureByName('font').bind;
drawString(w.Left+10, w.Top+5,w.title);

DrawChildren(w);
end;


procedure TGuiDrawer.DrawButton(w:TUiButton);
var i:integer;
    c:TUiContainer;
begin
if w.enabled then setColor(cWidget) else setColor(cDisabled);
if w.Focused then setColor(cFocused);

DrawRectangle(w.Left, w.Top, w.Width, w.Height);

fman.getTextureByName('font').bind;
if w.clicked then
  drawString(w.Left+8, w.Top+6,w.Caption)
else
  drawString(w.Left+8, w.Top+4,w.Caption);
end;


procedure TGuiDrawer.DrawEdit(w:TUiEdit);
var i:integer;
    c:TUiContainer;
    s:string;
begin
if w.enabled then setColor(cEdit) else setColor(cDisabled);
if w.Focused then setColor(cFocused);

DrawRectangle(w.Left, w.Top, w.Width, w.Height);

fman.getTextureByName('font').bind;

s:='';
if w.mask then 
 begin
 	 for i:=1 to length(w.text) do s:=s+'*';
 end
else 
 s:=w.text;

if w.focused then s:=s+'_';

drawString(w.Left+8, w.Top+4,s)
  
end;

procedure TGuiDrawer.DrawImage(w:TUiImage);
var i:integer;
begin
glColor3f(1, 1, 1);

glEnable(GL_TEXTURE_2D);
glBindTexture(GL_TEXTURE_2D, w.image);

DrawRectangle(w.Left, w.Top, w.Width, w.Height);
glDisable(GL_TEXTURE_2D);
end;

procedure TGuiDrawer.DrawList(w:TUiList);
var i:integer;
    c:TUiContainer;
begin
if w.enabled then setColor(cWidget) else setColor(cDisabled);
if w.Focused then setColor(cFocused);

DrawRectangle(w.Left, w.Top, w.Width, w.Height);


(*glColor3f(0,0,0);
DrawRectangleOutline(w.Left, w.Top, w.Width, w.height);
  *)

{ draw a bar behind selected text }
if (w.SelIndex>=0) and (not w.Simple) then 
  begin
   glColor3f(1,1,1);
   DrawRectangle(w.Left+1,w.Top+w.margintop+(w.SelIndex*12),w.Width-2,13);

  end;

fman.getTextureByName('font').bind;
for i:=0 to w.Strings.count-1 do
 drawString(w.Left+1, w.Top+w.margintop+3+i*12,w.strings[i]);

end;

procedure TGuiDrawer.DrawMultiTracker(w:TUiMultiTracker);
var i:integer;
    c:TUiContainer;
    realPos, logicPos, realWidth, logicWidth:integeR;
begin
if w.enabled then setColor(cWidget) else setColor(cDisabled);
if w.Focused then setColor(cFocused);

DrawRectangle(w.Left, w.Top, w.Width, w.Height);



{ draw the cursors }

logicWidth := w.max-w.min;
realwidth :=w.width-2*w.margin;

for i:=w.cursors.count-1 downto 0 do // reverse to draw first one on top of others
 begin
   logicPos := w.cursors[i].position - w.min;

   realpos:=(logicPos*realwidth div logicwidth) + w.margin;

   if w.selIndex = i then glColor3f(1,1,1) else glColor3f(0.6,0.6,0.6);
   
   DrawRectangle(w.Left+realpos-(w.CursorWidth div 2),w.Top,w.cursorwidth,w.height);
 end;
 
if w.selIndex>=0 then
 begin
  fman.getTextureByName('font').bind;
  drawString(w.Left+8, w.Top+4, inttostr(w.cursors[w.selindex].position)); 
 end;

end;


procedure TGuiDrawer.DrawTracker(w:TUiTracker);
var i:integer;
    c:TUiContainer;
    realPos, logicPos, realWidth, logicWidth:integeR;
begin
if w.enabled then setColor(cWidget) else setColor(cDisabled);
if w.Focused then setColor(cFocused);

DrawRectangle(w.Left, w.Top, w.Width, w.Height);

{ draw the cursor  }
logicPos := w.position - w.min;
logicWidth := w.max-w.min;
realwidth	:=w.width-2*w.margin;

realpos:=(logicPos*realwidth div logicwidth) + w.margin;

   glColor3f(1,1,1);
   DrawRectangle(w.Left+realpos-(w.CursorWidth div 2),w.Top,w.cursorwidth,w.height);

fman.getTextureByName('font').bind;
drawString(w.Left+8, w.Top+4, inttostr(w.position)); // @todo use w.caption

end;


procedure TGuiDrawer.DrawCheckBox(w:TUiCheckBox);
begin
if w.enabled then
  if w.Checked then glColor3f(0,1,0) else glColor3f(1,0,0)
else
  glColor3f(0.3,0.3,0.3);

DrawRectangle(w.Left, w.Top, 8, 8);

fman.getTextureByName('font').bind;
drawString(w.Left+10, w.Top+1,w.Caption);

end;
procedure TGuiDrawer.DrawBar(w:TUiBar);
begin
// @todo implement this method
end;

procedure TGuiDrawer.DrawLabel(w:TUiLabel);
begin
fman.getTextureByName('font').bind;
drawString(w.Left, w.Top,w.Caption);

end;


procedure TGuiDrawer.DrawWidget(w:TUiWidget);
var i:integer;
    c:TUiContainer;
begin
if w is TUiRollableWindow then DrawRollableWindow( TUiRollableWindow(w) )
else if w is TUiWindow then DrawWindow( TUiWindow(w) )
else if w is TUiButton then DrawButton( TUiButton(w) )
else if w is TUiLayered then DrawLayered( TUiLayered(w) )
else if w is TUiPage then DrawPage( TUiPage(w) )
else if w is TUiEdit then DrawEdit( TUiEdit(w) )
else if w is TUiList then DrawList( TUiList(w) )
else if w is TUiTracker then DrawTracker( TUiTracker(w) )
else if w is TUiMultiTracker then DrawMultiTracker( TUiMultiTracker(w) )
else if w is TUiImage then DrawImage( TUiImage(w) )
else if w is TUiCheckBox then DrawCheckBox( TUiCheckBox(w) )
else if w is TUiBar then DrawBar( TUiBar(w) )
else if w is TUiLabel then DrawLabel( TUiLabel(w) );

end;

end.

