{** This is a simple XML parser. It does handle all quite good,
except for initial <?XXX> or <!XXX> tags. It only handles ONE root tag.
Works for both linux and windows, for both delphi/kylix and freepascal. }
unit Nicola_xmlparser;

interface

(*

This unit was written by Nicola {MSX} Lugato.

http://msx.rules.it
nicola@lugato.net
nicola.lugato@gmail.com

It is released under the term of
The GNU General Public License as published by the Free
Software Foundation.

*)
uses SysUtils, Classes;

type

 {** The type of a node }
 TNodeType=(ndTag, ndComment, ndText, ndCData);

 TXMLNodes=class;


 {** this simple class is a single attribute (umm could be a record too..) }
 TXMLAttribute=class
  Key,Value:string;
 end;

 {** a list of attribute. Can use Attributes['test'] syntax to read test attribute.
     also, access by index is supported, and access by string returning the TXMLAttribute class }
 TXMLAttributes=class(TList)
  private
   function GetAttribute(index:string):string;
   procedure SetAttribute(index:string; o:string);
   function GetAttributeInt(index:integer):TXMLAttribute;
   procedure SetAttributeInt(index:integer; o:TXMLAttribute);
   function GetAttributeString(index:string):TXMLAttribute;
  public
   procedure ReadFrom(s:string);
   property Attribute[index:string]:string read GetAttribute write SetAttribute; default;
   property AttributeIndex[index:integer]:TXMLAttribute read GetAttributeInt Write SetAttributeInt;
   property AttributeObject[index:string]:TXMLAttribute read GetAttributeString;
   destructor Destroy;override;
 end;

 {** rappresent a XML node. Can by tag, text, cdata, comment ecc.. (check TNodeType) }
 TXMLNode=class
  private
   fText:string;
   function GetText:string;
   procedure SetText(t:string);

   function GetNumber:integer;
   procedure SetNumber(t:integer);

   procedure SkipTag(s:TStream);
   procedure LoadTag(s:TStream);
   procedure LoadComment(s:TStream);
   procedure LoadText(s:TStream);
   procedure LoadCData(s:TStream);

   procedure SaveTag(s:TStream);
   procedure SaveText(s:TStream);
   procedure SaveComment(s:TStream);
   procedure SaveCData(s:TStream);
  public

   nodetype:TNodeType;
   Name:string;
   Attributes:TXMLAttributes;
   Children:TXMLNodes;
   procedure Save(s:TStream);
   procedure Load(s:TStream);
   function Query(q:string):TXMLNode;
   procedure QueryRecursive(q:string; OutputList:TXMLNodes);
   constructor Create(ANodeName:string;ANodeType:TNodeType=ndTag);
   destructor Destroy;override;
   procedure AppendChildText(node,text:string);
   procedure AppendChildNumber(node:string; number:integer);
   property Text:string read GetText write SetText;
   property Number:integer read GetNumber write SetNumber;
 end;

 {** a set of Nodes. Can be queried for a single node, or nodes can be scanned from 0 to count-1 }
 TXMLNodes=class(TList)
  private
   function GetNode(index:integer):TXMLNode;
   procedure SetNode(index:integer; o:TXMLNode);
  public
   function Query(q:string):TXMLNode;
   property Node[index:integer]:TXMLNode read GetNode Write SetNode; default;
   destructor Destroy;override;
 end;

 {** a XML document. Remember that XML docs have only one root node.
     this just read/write the main node. }
 TXMLDocument=class
  Node:TXMLNode;
  constructor Create;
  destructor Destroy;override;
  function Query(q:string):TXMLNode;  
  procedure Load(s:TStream);overload;
  procedure Save(s:TStream);overload;
  procedure Load(Filename:string);overload;
  procedure Save(Filename:string);overload;
 end;


implementation

type
TTag=record
 name:string;
 tagtype:(ttOpen,ttClose,ttUnique);
 at:TXMLAttributes;
end;

TWhatsNext=(wnComment, wnCData, wnText, wnTagOpen, wnTagClose, wnTagUnique, wnSkippable);
const WNString:array[low(TWhatsNext)..high(TWhatsNext)] of string=('Comment','CData','text','tagopen','tagclose','tagunique', 'skippable');


function EncodeStantardEntities(const s: String): String;
begin
	Result := s;
	Result := StringReplace(Result, '&', '&amp;', [rfReplaceAll]);
	Result := StringReplace(Result, '<', '&lt;', [rfReplaceAll]);
	Result := StringReplace(Result, '>', '&gt;', [rfReplaceAll]);
	Result := StringReplace(Result, '"', '&quot;', [rfReplaceAll]);
	Result := StringReplace(Result, '''', '&apos;', [rfReplaceAll]);
end;

function DecodeStantardEntities(const s: String): String;
begin
	Result := s;
	Result := StringReplace(Result, '&apos;', '''', [rfReplaceAll]);
	Result := StringReplace(Result, '&quot;', '"', [rfReplaceAll]);
	Result := StringReplace(Result, '&gt;', '>', [rfReplaceAll]);
	Result := StringReplace(Result, '&lt;', '<', [rfReplaceAll]);
	Result := StringReplace(Result, '&amp;', '&', [rfReplaceAll]);
end;


function LastChar(s:string):char;
begin
if s='' then result:=#0
        else result:=s[length(s)];
end;


// simple shortcut for writing a string
procedure Write(s:TStream;st:string);
begin
if s.Write(st[1],length(st))<>length(st) then raise Exception.Create('Cannot write to stream!');
end;


{ look forward to see what's next }
function WhatsNext(s:Tstream):TWhatsNext;
var pos:longint;
    c,prev:char;
begin
pos:=s.Position;  // save position
if s.Read(c,1)<>1 then raise Exception.Create('Fine stream!');

// text
if c<>'<' then
begin
 result:=wnText;
 s.Position:=pos;  // restore position
 exit;
end;

if s.Read(c,1)<>1 then raise Exception.Create('Fine stream!');
// tagclose
if c='/' then
 begin
  result:=wnTagClose;
  s.Position:=pos;  // restore position
  exit;
 end;
// garbage
if c='?' then
 begin
  result:=wnSkippable;
  s.Position:=pos;  // restore position
  exit;
 end;
if c='!' then
begin

 if s.Read(c,1)<>1 then raise Exception.Create('Fine stream!');
 // cdata
 if c='[' then
  begin
   result:=wnCData;
   s.Position:=pos;  // restore position
   exit;
  end;
 // comment
 if c='-' then
  begin
   result:=wnComment;
   s.Position:=pos;  // restore position
   exit;
  end;
 result:=wnSkippable;
 s.Position:=pos;  // restore position
 exit;
end;
c:=#0;prev:=#0;
while c<>'>' do
begin
 prev:=c;
 if s.Read(c,1)<>1 then raise Exception.Create('Stream end while reading tag!');
end;
if prev='/' then result:=wnTagUnique else result:=wnTagOpen;
s.Position:=pos;  // restore position

end;

function ReadTag(s:TStream):TTag;
var c:char;
    t,name:string;
    i:integer;
begin
t:='';
if s.Read(c,1)<>1 then raise Exception.Create('End stream while tag expected!');
if c<>'<' then raise Exception.Create('No tag while expected!');

if s.Read(c,1)<>1 then raise Exception.Create('End stream after "<"!');
while c<>'>' do
begin
 t:=t+c;
 if s.Read(c,1)<>1 then raise Exception.Create('End stream before finding ">" !');
end;
// now t has all but "<" and ">": attributes, "/"s, ecc.

// let's see what kind of tag it is
result.tagtype:=ttOpen;
if lastchar(t)='/' then
 begin
  Result.tagtype:=ttUnique;
  t:=copy(t,1,length(t)-1);  // discard "/"
 end
else
 if t[1]='/' then
  begin
   Result.tagtype:=ttClose;
   t:=copy(t,2,length(t)-1); // discard "/"
  end;
// now t contains name and attribute
i:=1; name:='';
while (i<=length(t)) and (t[i]<>' ') do
 begin
  name:=name+t[i];
  inc(i);
 end;
// now name is ok, let's check for attributes
result.at:=nil;
if i<length(t) then
 begin
  inc(i); // skip the space
  t:=copy(t,i,length(t)-i+1);
  Result.at:=TXMLAttributes.Create;
  Result.at.ReadFrom(t);
 end;
result.name:=name;
end;

function TXMLNodes.GetNode(index:integer):TXMLNode;
begin
 result:=TXMLNode(items[index]);
end;
procedure TXMLNodes.SetNode(index:integer; o:TXMLNode);
begin
 items[index]:=o;
end;

constructor TXMLNode.Create(ANodeName:string;ANodeType:TNodeType=ndTag);
begin
Attributes:=TXMLAttributes.Create;
Children:=TXMLNodes.create;
ftext:='';
nodetype:=ANodeType;
case nodetype of
 ndTag: name:=ANodeName;
 ndComment: name:='#comment';
 ndText: name:='#text';
 ndCData: name:='#cdata';
end;
end;


constructor TXMLDocument.Create;
begin
Node:=TXMLNode.Create('Main',ndTag);
end;

procedure TXMLDocument.Save(s:TStream);
begin
Node.save(s);
end;

procedure TXMLDocument.Load(s:TStream);
begin
Node.Free;
node:=TXMLNode.Create('a',ndText);
while node.nodetype<>ndTag do
 begin
  node.Load(s);
 end;
end;

procedure TXMLNode.Load(s:TStream);
begin
 case WhatsNext(s) of
  wnTagOpen,
  wnTagUnique: LoadTag(s);
  wnComment: LoadComment(s);
  wnCData: LoadCData(s);
  wnText: LoadText(s);
  wnSkippable:
    begin
      SkipTag(s);
      load(s);
    end;
  wnTagClose: raise Exception.create('Close tag in wrong place!');
 end;
end;


procedure TXMLNode.LoadTag(s:TStream);
var t:string;
    tag:TTag;
    x:TXMLNode;
    w:TWhatsNext;
begin
nodetype:=ndTag;
tag:=ReadTag(s);
t:=tag.name;
if tag.at<>nil then
 begin
  Attributes.free;
  Attributes:=tag.at;
 end;
Name:=t;
if tag.tagtype=ttClose then raise exception.create('Close tag find in wrong place!');
if tag.tagtype=ttUnique then exit;

while true do
 begin
  w:=WhatsNext(s);
 case w of
  wnTagOpen, wnTagUnique:
   begin
    x:=TXMLNode.create('',ndTag);
    x.LoadTag(s);
    Children.Add(x);
   end;
  wnComment:
   begin
    x:=TXMLNode.create('',ndTag);
    x.LoadComment(s);
    Children.Add(x);
   end;
  wnCData:
   begin
    x:=TXMLNode.create('',ndTag);
    x.LoadCData(s);
    Children.Add(x);
   end;
  wnText:
   begin
    x:=TXMLNode.create('',ndTag);
    x.LoadText(s);
    Children.Add(x);
   end;
  wnTagClose:
   begin
    tag:=ReadTag(s);
    if tag.name<>name then raise exception.create('Close tag different from open! '+tag.name+'<>'+name);
    break;
   end;
 end;
end;


end;


procedure TXMLNode.LoadComment(s:TStream);
var c,n:char;
    ancora:boolean;
    pos:longint;
begin
nodetype:=ndComment;
Name:='#comment';
c:=#0;ancora:=true;
text:='';
while ancora do
 begin
  if s.Read(c,1)<>1 then raise Exception.Create('Fine stream!');
  if c='-' then
   begin
    pos:=s.Position;
    if s.Read(n,1)<>1 then raise Exception.Create('Fine stream!');
     if n='-' then
      begin
       if s.Read(n,1)<>1 then raise Exception.Create('Fine stream!');
       if n='>' then
        begin
         ancora:=false;
        end;
      end;
    if ancora then s.Position:=pos;
   end;
  if ancora then
   begin
    text:=text+c;
   end;

 end;
if copy(text,1,4)<>'<!--' then raise Exception.Create('Error reading comment!');
text:=copy(text,5,length(text)-4);
end;

procedure TXMLNode.LoadCData(s:TStream);
var c,n:char;
    ancora:boolean;
    pos:longint;
begin
nodetype:=ndCdata;
Name:='#cdata';
c:=#0;ancora:=true;
text:='';
while ancora do
 begin
  if s.Read(c,1)<>1 then raise Exception.Create('Fine stream!');
  if c=']' then
   begin
    pos:=s.Position;
    if s.Read(n,1)<>1 then raise Exception.Create('Fine stream!');
     if n=']' then
      begin
       if s.Read(n,1)<>1 then raise Exception.Create('Fine stream!');
       if n='>' then
        begin
         ancora:=false;
        end;
      end;
    if ancora then s.Position:=pos;
   end;
  if ancora then
   begin
    text:=text+c;
   end;
 end;
if copy(text,1,9)<>'<![CDATA[' then raise Exception.Create('Error reading cdata!');
text:=copy(text,10,length(text)-9);
end;

procedure TXMLNode.SkipTag(s:TStream);
var ancora:boolean;
    c:char;
begin
ancora:=true;
while ancora do
 begin
  if s.Read(c,1)<>1 then raise Exception.Create('Fine stream!');
  if c='>' then
   begin
    ancora:=false;
    s.Position:=s.Position;
   end;
 end;

end;

procedure TXMLNode.LoadText(s:TStream);
var c:char;
    ancora:boolean;
begin
nodetype:=ndText;
Name:='#text';
text:='';
c:=#0;ancora:=true;
while ancora do
 begin
  if s.Read(c,1)<>1 then raise Exception.Create('Fine stream!');
  if c='<' then
   begin
    ancora:=false;
    s.Position:=s.Position-1;
   end;
  if ancora then
   text:=text+c;
 end;
text:=DecodeStantardEntities(text);
end;

destructor TXMLNode.Destroy;
begin
Children.Destroy;
Attributes.Destroy;
inherited Destroy;
end;

destructor TXMLNodes.Destroy;
begin
while count>0 do
 begin
  Node[0].Free;
  Delete(0);
 end;
inherited Destroy;
end;


destructor TXMLDocument.Destroy;
begin
Node.Destroy;
inherited Destroy;
end;

function TXMLAttributes.GetAttributeInt(index:integer):TXMLAttribute;
begin
result:=TXMLAttribute(items[index]);
end;
procedure TXMLAttributes.SetAttributeInt(index:integer; o:TXMLAttribute);
begin
items[index]:=o;
end;

function TXMLAttributes.GetAttributeString(index:string):TXMLAttribute;
var i:integer;
begin
for i:=0 to count-1 do
 if AttributeIndex[i].Key=index then
  begin
   result:=AttributeIndex[i];
   exit;
  end;
result:=nil;
end;

function TXMLAttributes.GetAttribute(index:string):string;
var o:TXMLAttribute;
begin
o:=AttributeObject[index];
if o<>nil then result:=o.value else result:='';
end;

procedure TXMLAttributes.SetAttribute(index:string; o:string);
var a:TXMLAttribute;
begin
a:=AttributeObject[index];
if a<>nil then a.value:=o
 else
  begin
   a:=TXMLAttribute.Create;
   a.Key:=index;
   a.Value:=o;
   add(a);
  end;

end;

procedure TXMLAttributes.ReadFrom(s:string);

procedure ReadOne(var i:integer;s:string);
var a:TXMLAttribute;
    sep:char;
begin
 // legge un singolo attributo
 a:=TXMLAttribute.Create;
 while s[i]<>'=' do
  begin
   a.Key:=a.Key + s[i];
   inc(i);
  end;
 a.Key:=trim(a.Key);
 inc(i);
 sep:=s[i];
 if (sep<>'"') and (sep<>'''') then raise Exception.Create('Expected " or '' as attribute value begin! Found:'+sep);
 inc(i);
 while s[i]<>sep do
  begin
   a.value:=a.value + s[i];
   inc(i);
  end;
 Add(a);
end;

var i:integer;
begin
i:=1;
while i<length(s) do
 begin
  ReadOne(i,s);
  inc(i,2);
 end;

end;

destructor TXMLAttributes.Destroy;
begin
 while count>0 do
  begin
   AttributeIndex[0].Free;
   Delete(0);
  end;
inherited Destroy;
end;

procedure TXMLNode.Save(s:TStream);
begin
case nodetype of
 ndTag:SaveTag(s);
 ndComment:SaveComment(s);
 ndText:SaveText(s);
 ndCData:SaveCData(s);
end;
end;


procedure TXMLNode.SaveTag(s:TStream);
var i:integer;
    att, t:string;
begin
att:='';
for i:=0 to Attributes.Count-1 do
 att:=att+' '+Attributes.AttributeIndex[i].Key+'="'+Attributes.AttributeIndex[i].Value+'"';

t:=Name+att;  // name and attributes

if Children.Count=0 then
 begin
  // void tag
  Write(s,'<'+t+'/>');
  exit;
 end;

Write(s,'<'+t+'>');
for i:=0 to Children.Count-1 do
 children[i].Save(s);
Write(s,'</'+name+'>');
end;
procedure TXMLNode.SaveText(s:TStream);
var x:string;
begin
x:=EncodeStantardEntities(text);
Write(s,x);
end;
procedure TXMLNode.SaveComment(s:TStream);
var x:string;
begin
x:='<!--'+text+'-->';
Write(s,x);
end;
procedure TXMLNode.SaveCData(s:TStream);
var x:string;
begin
x:='<![CDATA['+text+']]>';
Write(s,x);
end;


function TXMLNode.Query(q:string):TXMLNode;
begin
result:=Children.Query(q);
end;


function TXMLNodes.Query(q:string):TXMLNode;
var i:integer;
begin
for i:=0 to Count-1 do
 if Node[i].Name=q then
  begin
   result:=Node[i];
   exit;
  end;
result:=nil;
end;


function TXMLDocument.Query(q:string):TXMLNode;
begin
if node.Name=q then result:=node else result:=nil;
end;


procedure TXmlDocument.Load(Filename:string);
var fs:TFileStream;
begin
  fs:=TFileStream.Create(filename,fmOpenRead);
  Load(fs);
  fs.Free;
end;

procedure TXmlDocument.Save(Filename:string);
var fs:TFileStream;
begin
fs:=TFileStream.Create(filename,fmCreate);
Save(fs);
fs.Free;
end;


function TXmlNode.GetNumber:integer;
begin
result:=strtoint(gettext);
end;
procedure TXmlNode.SetNumber(t:integer);
begin
SetText(inttostr(t));
end;

function TXmlNode.GetText:string;
begin
if nodetype<>ndTag then
 begin
  result:=fText;
  exit;
 end;
if (Children.Count=1) and (Children[0].nodetype<>ndTag) then
 begin
  result:=Children[0].fText;
  exit;
 end;
result:='';
end;

procedure TXmlNode.SetText(t:string);
var n:TXMLNode;
begin
if nodetype<>ndTag then
 begin
  fText:=t;
  exit;
 end;

// if tag contains a text children, then the user
// it's tring to write it
if (Children.Count=1) and (Children[0].nodetype<>ndTag) then
 begin
  Children[0].fText:=t;
  exit;
 end;

// if tag contains no children, then we must provide
// a child to write to
if (Children.Count=0) then
 begin
  n:=TXMLNode.Create('#text',ndText);
  n.fText:=t;
  Children.Add(n);
  exit;
 end;

raise exception.create('Can''t put text in a tag without text.');
end;


procedure TXmlNode.AppendChildText(node,text:string);
var n:TXMLNode;
begin
n:=TXMLNode.create(node,ndTag);
n.Text:=text;
Children.Add(n);
end;
procedure TXmlNode.AppendChildNumber(node:string; number:integer);
begin
AppendChildText(node, inttostr(number));
end;


procedure TXMLNode.QueryRecursive(q:string; OutputList:TXMLNodes);
var i:integer;
begin
for i:=0 to Children.Count-1 do
 begin
  if Children[i].Name=q then OutputList.Add(Children[i]);
  Children[i].QueryRecursive(q, OutputList);
 end;

end;

end.
