unit soundunit;

(*

This unit was written by Nicola {MSX} Lugato.

http://msx.rules.it
nicola@lugato.net
nicola.lugato@gmail.com

It is released under the term of
The GNU General Public License as published by the Free
Software Foundation.


Description:

This unit implements the sound system.
It is actually almost empty, becouse i have yet to choose
a sound lib to use.
Some tests have been done with fmod, but i don't like it's license
and it's not being open source.

Changelog:

2004-11-06: changelog started

*)


interface

uses Classes, openal, types3d, msxmath;
// fmod, fmodtypes, fmoderrors;

const alorigin: array [0..2] of TALfloat= ( 0.0, 0.0, 0.0 );
      listenerorientation: array [0..5] of TALfloat= ( 0.0, 0.0, -1.0, 0.0, 1.0, 0.0);

type

  // useful compatibility with ode vectors
  // but works only as long as ODE single DLL is used
  TAlVector3 = TVector;
  // use this if you dont' need ODE
  // TAlVector3 = array [0..2] of TALfloat;

  TSound = class
    name:string;
    buffer:TALInt;

    format: TALEnum;
    size: TALSizei;
    freq: TALSizei;
    loop: TALInt;
    data: TALVoid;
    constructor Create(filename:string);
    destructor Destroy; override;
  end;

  TSource = class
   private
    source : TALint;
    function getPitch():single;
    procedure setPitch(v:single);
    function getGain():single;
    procedure setGain(v:single);
    procedure setPosition(v:TAlVector3);
    procedure setVelocity(v:TAlVector3);
    procedure setLoop(v:boolean);
    function getLoop():boolean;
   public
    property Pitch:single read getPitch write setPitch;
    property Gain:single read getGain write setGain;
    property Loop:boolean read getLoop write setLoop;
    property Position:TAlVector3 write setPosition;
    property Velocity:TAlVector3 write setVelocity;
    procedure Play;
    destructor Destroy; override;
    constructor Create(sound:TSound);
  end;

  TSoundList = class(TList)
      function getSound(index:integer):TSound;
      property Sound[index:integer]:TSound read getSound; default;
       function getSoundByName(name:string):TSound;
     end;


function InitSound:boolean;
procedure setListenerOrientation(dir, up:TAlVector3);
procedure setListenerPosition(v:TAlVector3);
procedure setListenerVelocity(v:TAlVector3);

implementation

procedure setListenerPosition(v:TAlVector3);
begin
 AlListenerfv ( AL_POSITION, @v);
end;
procedure setListenerVelocity(v:TAlVector3);
begin
 AlListenerfv ( AL_VELOCITY, @v);
end;

procedure setListenerOrientation(dir, up:TAlVector3);
var  listenerori: array [0..5] of TALfloat;
begin
listenerori[0]:=dir.x;
listenerori[1]:=dir.y;
listenerori[2]:=dir.z;
listenerori[3]:=up.x;
listenerori[4]:=up.y;
listenerori[5]:=up.z;
AlListenerfv ( AL_ORIENTATION, @listenerori);
end;

destructor TSound.Destroy;
begin
  if alIsBuffer ( buffer) then
     AlDeleteBuffers(1, @buffer);
end;

constructor TSound.Create(filename:string);
begin
  writeln('loaing '+filename);
  AlGenBuffers(1, @buffer);
  AlutLoadWavFile(filename, format, data, size, freq, loop);
  AlBufferData(buffer, format, data, size, freq);
  AlutUnloadWav(format, data, size, freq);
  writeln('done loaing');
end;

function InitSound:boolean;
begin
  // warning: make cross platform and handle errors!
  InitOpenAL('libopenal.so.0');
  // AlutInit();

  AlListenerfv ( AL_POSITION, @alorigin);
  AlListenerfv ( AL_VELOCITY, @alorigin);
  AlListenerfv ( AL_ORIENTATION, @listenerorientation);
  alDistanceModel( AL_INVERSE_DISTANCE );


  result:=TRUE;
end;

function TSoundList.getSound(index:integer):TSound;
begin
result:=TSound(items[index]);
end;

function TSoundList.getSoundByName(name:string):TSound;
var i:integer;
begin
 for i:=0 to count-1 do
  if Sound[i].name=name then
  begin
   result:=Sound[i];
   exit;
  end;
 result:=nil;
end;

function TSource.getPitch():single;
begin
alGetSourcef( source, AL_PITCH, @result );
end;

procedure TSource.setPitch(v:single);
begin
AlSourcef ( source, AL_PITCH, v );
end;


function TSource.getGain():single;
begin
alGetSourcef( source, AL_Gain, @result );
end;

procedure TSource.setGain(v:single);
begin
AlSourcef ( source, AL_Gain, v );
end;

procedure TSource.setPosition(v:TAlVector3);
begin
AlSourcefv ( source, AL_POSITION, @v);
end;

procedure TSource.setVelocity(v:TAlVector3);
begin
AlSourcefv ( source, AL_VELOCITY, @v);
end;

destructor TSource.Destroy;
begin
if alIsSource(source) then
  AlDeleteSources(1, @source);
end;

constructor TSource.Create(sound:TSound);
begin
  AlGenSources(1, @source);
  AlSourcei ( source, AL_BUFFER, sound.buffer);
  AlSourcef ( source, AL_PITCH, 1.0 );
  AlSourcef ( source, AL_GAIN, 1.0 );
//  AlSourcef ( source, AL_REFERENCE_DISTANCE, 50 );
  AlSourcefv ( source, AL_POSITION, @alorigin);
  AlSourcefv ( source, AL_VELOCITY, @alorigin);
  AlSourcei ( source, AL_LOOPING, AL_FALSE);
end;

procedure TSource.setLoop(v:boolean);
var b:integer;
begin
if v then b:=AL_TRUE else b:=AL_FALSE;
AlSourcei ( source, AL_LOOPING, b);
end;

function TSource.getLoop():boolean;
var b:integer;
begin
  alGetSourcei( source, AL_LOOPING, @b);
  result:=b=AL_TRUE;
end;

procedure TSource.Play;
begin
 AlSourcePlay(source);
end;


end.
