unit ServerUnit;

interface

uses
  SysUtils, Classes, networkunit, logger;


type

TServerException = class(Exception)
end;

TPacketServer = class;


TEventType=(eConnection, eDisconnection, eMessage);

TOnEvent = procedure (Sender:TPacketServer; conn:TSocket; EventType:TEventType; text:string) of object;

TPacketServer=class
 private
  fMaxBufferSize:cardinal;
  fServerSocket:TSocket;
  fSocketSet:TSocketSet;
  fSocketList:TList;
  function handleNewConnection:boolean;
  procedure handleNewData;
  procedure CallEvent(conn:TSocket; EventType:TEventType; text:string);
 public
  OnEvent:TOnEvent;
  constructor Create(maxclient:cardinal);
  
  procedure Start(port:integer; MaxBufferSize:cardinal);
  procedure Execute;
  procedure Close;
  destructor Destroy;override;
end;


implementation

constructor TPacketServer.Create(maxclient:cardinal);
begin
fServerSocket:=TSocket.create;
fSocketList:=TList.create;
fSocketSet:=TSocketSet.Create(maxclient);
end;

destructor TPacketServer.Destroy;
begin
fServerSocket.free;
fSocketList.free;
fSocketSet.free;
end;

procedure TPacketServer.Start(port:integer; MaxBufferSize:cardinal);
begin
  fMaxBufferSize:=MaxBufferSize; // @todo use this to limit string size
  // start listening
  if not fServerSocket.Listen(50401) then
    raise TServerException.Create('ERROR SERVER '+getLastNetworkError);
  // add server socket so watched socket
  fSocketSet.Add(fServerSocket);
  Execute;
end;

function TPacketServer.handleNewConnection:boolean;
var s:TSocket;
begin
s:=fServerSocket.Accept;
if s=nil then
 // the server closed!
 result:=false
else
 begin
   // a new connection arrived
   fSocketList.add(s);
   fSocketSet.Add(s);
   CallEvent(s, eConnection, '');
   result:=true;
 end;
end;

procedure TPacketServer.handleNewData;
var i:integer;
    sock:TSocket;
    s:string;
begin
// first search socket with data
for i:=fSocketList.count-1 downto 0 do
  begin
    sock:=TSocket(fSocketList[i]);
    if fSocketSet.SocketActive( sock ) then
      begin
        // this socket is active!
        if sock.Recv(s) then
          begin
            // incoming string!
            CallEvent(sock, eMessage, s);
          end
        else
          begin
            // disconnection
            fSocketList.Remove(sock);
            fSocketSet.Remove(sock);
            CallEvent(sock, eDisconnection, '');
            sock.free;
          end;
        
      end;
  end;
end;

Procedure TPacketServer.Execute;
var c:TSocket;
begin
 while fSocketSet.Activity(true) do
   begin
    if fSocketSet.SocketActive(fServerSocket) then
      begin
        // incoming connection or server closing
        if not handleNewConnection then exit;
      end
    else
      begin
        // some socket has data to read or disconnected
        handleNewData;
      end;
   end;
   

end;

procedure TPacketServer.Close;
begin
Log('Closing server!');
fserversocket.Close;
end;

procedure TPacketServer.CallEvent(conn:TSocket; EventType:TEventType; text:string);
begin

if assigned(OnEvent) then
 onEvent(self, conn, eventtype, text);

end;

end.
